import 'package:flutter/services.dart';

/// Date input formatter.
abstract class DateFieldFormatter extends TextInputFormatter {
  late final String _separator;

  DateFieldFormatter(String separator) {
    _separator = separator;
  }

  @override
  TextEditingValue formatEditUpdate(
      final TextEditingValue oldValue, final TextEditingValue newValue) {
    /// If cursor doesn't at the end then take the old value.
    final String value = newValue.selection.end == newValue.text.length
        ? newValue.text.replaceAll(_separator, '')
        : oldValue.text.replaceAll(_separator, '');

    /// Construct formatted
    final StringBuffer formatted = StringBuffer();
    for (int index = 0; index < value.length; index++) {
      if (index == firstSeparatorIndex || index == secondSeparatorIndex) {
        formatted.write(_separator);
      }
      formatted.write(value[index]);
    }
    return TextEditingValue(
        selection: TextSelection.collapsed(offset: formatted.length),
        text: formatted.toString());
  }

  int get firstSeparatorIndex;

  int get secondSeparatorIndex;
}
