import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_custom_form_component/widget/field/mixin.dart';
import 'package:flutter_custom_form_component/widget/field/money/formatter.dart';

/// Text field supports money format.
class MoneyFieldWidget extends StatelessWidget with FieldWidgetMixin {
  late final String _decimalSeparator, _thousandSeparator;
  late final String? _hint;
  late final Function(double? value)? _onChanged;
  late final String? _symbol;

  MoneyFieldWidget(
      {String? hint,
      bool isUsingPeriodInThousand = true,
      Function(double? value)? onChanged,
      String? symbol,
      Key? key})
      : super(key: key) {
    _hint = hint;
    _onChanged = onChanged;
    _symbol = symbol;

    /// Using '.' for separating thousands and ',' for separating decimal. Or the opposite.
    _decimalSeparator = isUsingPeriodInThousand ? '.' : ',';
    _thousandSeparator = isUsingPeriodInThousand ? ',' : '.';
  }

  @override
  Widget build(final BuildContext context) => container(
      textField: TextField(
          decoration:
              InputDecoration(border: InputBorder.none, hintText: _hint),
          inputFormatters: [
            FilteringTextInputFormatter.allow(
                RegExp('[0-9$_decimalSeparator]')),
            MoneyFormatter(
                decimalSeparator: _decimalSeparator,
                symbol: _symbol,
                thousandSeparator: _thousandSeparator)
          ],
          keyboardType: Platform.isAndroid
              ? const TextInputType.numberWithOptions(decimal: true)
              : TextInputType.text,
          onChanged:

              /// Listener
              (String text) {
            text = text
                .replaceAll(_thousandSeparator, '')
                .replaceFirst(_decimalSeparator, '.');
            if (_symbol != null) {
              text = text.replaceFirst(_symbol!, '');
            }
            _onChanged?.call(double.tryParse(text));
          }));
}
