import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_custom_form_component/widget/field/field.dart';
import 'package:flutter_custom_form_component/widget/field/mixin.dart';

/// Text field supports name format.
class NameFieldWidget extends FieldWidget {
  late final String _initialValue;
  late final Function(String value)? _onChanged;

  NameFieldWidget(
      {TextInputAction action = TextInputAction.next,
      String? hint,
      String initialValue = '',
      bool isEnabled = true,
      Key? key,
      Function(String value)? onChanged,
      String? value})
      : super(action: action, hint: hint, isEnabled: isEnabled, key: key) {
    _initialValue = initialValue;
    _onChanged = onChanged;
  }

  @override
  State<StatefulWidget> createState() => _State();
}

class _State extends State<NameFieldWidget> with FieldWidgetMixin {
  final TextEditingController _controller = TextEditingController();

  @override
  Widget build(final BuildContext context) => container(
      textField: TextField(
          controller: _controller,
          decoration:
              InputDecoration(border: InputBorder.none, hintText: widget.hint),
          enabled: widget.isEnabled,
          inputFormatters: [
            FilteringTextInputFormatter.allow(RegExp('[a-z A-Z]'))
          ],
          keyboardType: TextInputType.name,
          textInputAction: widget.action));

  void _controllerListener() {
    widget._onChanged?.call(_controller.text);
  }

  @override
  void dispose() {
    _controller.removeListener(_controllerListener);
    _controller.dispose();
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    _controller.addListener(_controllerListener);
    _controller.text = widget._initialValue;
  }
}
